@@ -1,45 +1,29 @@
+// Copyright (C) 2004-2022 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
+// CA 94945, U.S.A., +1(415)492-9861, for further information.
+
 #ifndef MUPDF_PDF_EVENT_H
 #define MUPDF_PDF_EVENT_H
 
-enum
-{
-	HOTSPOT_POINTER_DOWN = 0x1,
-	HOTSPOT_POINTER_OVER = 0x2
-};
-
-/* Types of UI event */
-enum
-{
-	PDF_EVENT_TYPE_POINTER,
-};
-
-/* Types of pointer event */
-enum
-{
-	PDF_POINTER_DOWN,
-	PDF_POINTER_UP,
-};
-
-/*
-	UI events that can be passed to an interactive document.
-*/
-typedef struct pdf_ui_event_s
-{
-	int etype;
-	union
-	{
-		struct
-		{
-			int ptype;
-			fz_point pt;
-		} pointer;
-	} event;
-} pdf_ui_event;
-
-/*
-	pdf_init_ui_pointer_event: Set up a pointer event
-*/
-void pdf_init_ui_pointer_event(pdf_ui_event *event, int type, float x, float y);
+#include "mupdf/pdf/document.h"
 
 /*
 	Document events: the objects via which MuPDF informs the calling app
@@ -48,18 +32,7 @@ void pdf_init_ui_pointer_event(pdf_ui_event *event, int type, float x, float y);
 	callback.
 */
 
-/*
-	pdf_pass_event: Pass a UI event to an interactive
-	document.
-
-	Returns a boolean indication of whether the ui_event was
-	handled. Example of use for the return value: when considering
-	passing the events that make up a drag, if the down event isn't
-	accepted then don't send the move events or the up event.
-*/
-int pdf_pass_event(fz_context *ctx, pdf_document *doc, pdf_page *page, pdf_ui_event *ui_event);
-
-struct pdf_doc_event_s
+struct pdf_doc_event
 {
 	int type;
 };
@@ -72,21 +45,21 @@ enum
 	PDF_DOCUMENT_EVENT_MAIL_DOC,
 	PDF_DOCUMENT_EVENT_SUBMIT,
 	PDF_DOCUMENT_EVENT_EXEC_MENU_ITEM,
-	PDF_DOCUMENT_EVENT_EXEC_DIALOG
 };
 
 /*
-	pdf_set_doc_event_callback: set the function via which to receive
+	set the function via which to receive
 	document events.
 */
-void pdf_set_doc_event_callback(fz_context *ctx, pdf_document *doc, pdf_doc_event_cb *event_cb, void *data);
+void pdf_set_doc_event_callback(fz_context *ctx, pdf_document *doc, pdf_doc_event_cb *event_cb, pdf_free_doc_event_data_cb *free_event_data_cb, void *data);
+void *pdf_get_doc_event_callback_data(fz_context *ctx, pdf_document *doc);
 
 /*
 	The various types of document events
 */
 
 /*
-	pdf_alert_event: details of an alert event. In response the app should
+	details of an alert event. In response the app should
 	display an alert dialog with the buttons specified by "button_type_group".
 	If "check_box_message" is non-NULL, a checkbox should be displayed in
 	the lower-left corned along with the message.
@@ -97,10 +70,12 @@ void pdf_set_doc_event_callback(fz_context *ctx, pdf_document *doc, pdf_doc_even
 */
 typedef struct
 {
+	pdf_document *doc;
 	const char *message;
 	int icon_type;
 	int button_group_type;
 	const char *title;
+	int has_check_box;
 	const char *check_box_message;
 	int initially_checked;
 	int finally_checked;
@@ -136,41 +111,21 @@ enum
 };
 
 /*
-	pdf_access_alert_event: access the details of an alert event
+	access the details of an alert event
 	The returned pointer and all the data referred to by the
 	structure are owned by mupdf and need not be freed by the
 	caller.
 */
-pdf_alert_event *pdf_access_alert_event(fz_context *ctx, pdf_doc_event *event);
+pdf_alert_event *pdf_access_alert_event(fz_context *ctx, pdf_doc_event *evt);
 
 /*
-	pdf_access_exec_menu_item_event: access the details of am execMenuItem
+	access the details of am execMenuItem
 	event, which consists of just the name of the menu item
 */
-const char *pdf_access_exec_menu_item_event(fz_context *ctx, pdf_doc_event *event);
+const char *pdf_access_exec_menu_item_event(fz_context *ctx, pdf_doc_event *evt);
 
 /*
-	pdf_submit_event: details of a submit event. The app should submit
-	the specified data to the specified url. "get" determines whether
-	to use the GET or POST method.
-*/
-typedef struct
-{
-	char *url;
-	char *data;
-	int data_len;
-	int get;
-} pdf_submit_event;
-
-/*
-	pdf_access_submit_event: access the details of a submit event
-	The returned pointer and all data referred to by the structure are
-	owned by mupdf and need not be freed by the caller.
-*/
-pdf_submit_event *pdf_access_submit_event(fz_context *ctx, pdf_doc_event *event);
-
-/*
-	pdf_launch_url_event: details of a launch-url event. The app should
+	details of a launch-url event. The app should
 	open the url, either in a new frame or in the current window.
 */
 typedef struct
@@ -180,14 +135,14 @@ typedef struct
 } pdf_launch_url_event;
 
 /*
-	pdf_access_launch_url_event: access the details of a launch-url
+	access the details of a launch-url
 	event. The returned pointer and all data referred to by the structure
 	are owned by mupdf and need not be freed by the caller.
 */
-pdf_launch_url_event *pdf_access_launch_url_event(fz_context *ctx, pdf_doc_event *event);
+pdf_launch_url_event *pdf_access_launch_url_event(fz_context *ctx, pdf_doc_event *evt);
 
 /*
-	pdf_mail_doc_event: details of a mail_doc event. The app should save
+	details of a mail_doc event. The app should save
 	the current state of the document and email it using the specified
 	parameters.
 */
@@ -201,16 +156,12 @@ typedef struct
 	const char *message;
 } pdf_mail_doc_event;
 
-/*
-	pdf_access_mail_doc_event: access the details of a mail-doc event.
-*/
-pdf_mail_doc_event *pdf_access_mail_doc_event(fz_context *ctx, pdf_doc_event *event);
+pdf_mail_doc_event *pdf_access_mail_doc_event(fz_context *ctx, pdf_doc_event *evt);
 
-void pdf_event_issue_alert(fz_context *ctx, pdf_document *doc, pdf_alert_event *event);
+void pdf_event_issue_alert(fz_context *ctx, pdf_document *doc, pdf_alert_event *evt);
 void pdf_event_issue_print(fz_context *ctx, pdf_document *doc);
 void pdf_event_issue_exec_menu_item(fz_context *ctx, pdf_document *doc, const char *item);
-void pdf_event_issue_exec_dialog(fz_context *ctx, pdf_document *doc);
 void pdf_event_issue_launch_url(fz_context *ctx, pdf_document *doc, const char *url, int new_frame);
-void pdf_event_issue_mail_doc(fz_context *ctx, pdf_document *doc, pdf_mail_doc_event *event);
+void pdf_event_issue_mail_doc(fz_context *ctx, pdf_document *doc, pdf_mail_doc_event *evt);
 
 #endif
