@@ -1,19 +1,43 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
+// CA 94945, U.S.A., +1(415)492-9861, for further information.
+
 #ifndef PDF_INTERPRET_H
 #define PDF_INTERPRET_H
 
 #include "mupdf/pdf/font.h"
 #include "mupdf/pdf/resource.h"
+#include "mupdf/pdf/document.h"
 
-typedef struct pdf_csi_s pdf_csi;
-typedef struct pdf_gstate_s pdf_gstate;
-typedef struct pdf_processor_s pdf_processor;
+typedef struct pdf_gstate pdf_gstate;
+typedef struct pdf_processor pdf_processor;
 
 void *pdf_new_processor(fz_context *ctx, int size);
+pdf_processor *pdf_keep_processor(fz_context *ctx, pdf_processor *proc);
 void pdf_close_processor(fz_context *ctx, pdf_processor *proc);
 void pdf_drop_processor(fz_context *ctx, pdf_processor *proc);
 
-struct pdf_processor_s
+struct pdf_processor
 {
+	int refs;
 	void (*close_processor)(fz_context *ctx, pdf_processor *proc);
 	void (*drop_processor)(fz_context *ctx, pdf_processor *proc);
 
@@ -84,9 +108,9 @@ struct pdf_processor_s
 
 	/* text showing */
 	void (*op_TJ)(fz_context *ctx, pdf_processor *proc, pdf_obj *array);
-	void (*op_Tj)(fz_context *ctx, pdf_processor *proc, char *str, int len);
-	void (*op_squote)(fz_context *ctx, pdf_processor *proc, char *str, int len);
-	void (*op_dquote)(fz_context *ctx, pdf_processor *proc, float aw, float ac, char *str, int len);
+	void (*op_Tj)(fz_context *ctx, pdf_processor *proc, char *str, size_t len);
+	void (*op_squote)(fz_context *ctx, pdf_processor *proc, char *str, size_t len);
+	void (*op_dquote)(fz_context *ctx, pdf_processor *proc, float aw, float ac, char *str, size_t len);
 
 	/* type 3 fonts */
 	void (*op_d0)(fz_context *ctx, pdf_processor *proc, float wx, float wy);
@@ -140,7 +164,7 @@ struct pdf_processor_s
 	int hidden;
 };
 
-struct pdf_csi_s
+typedef struct
 {
 	/* input */
 	pdf_document *doc;
@@ -158,34 +182,128 @@ struct pdf_csi_s
 	pdf_obj *obj;
 	char name[256];
 	char string[256];
-	int string_len;
+	size_t string_len;
 	int top;
 	float stack[32];
-};
+} pdf_csi;
 
 /* Functions to set up pdf_process structures */
 
 pdf_processor *pdf_new_run_processor(fz_context *ctx, fz_device *dev, fz_matrix ctm, const char *usage, pdf_gstate *gstate, fz_default_colorspaces *default_cs, fz_cookie *cookie);
 
+/*
+	Create a buffer processor.
+
+	This collects the incoming PDF operator stream into an fz_buffer.
+
+	buffer: The (possibly empty) buffer to which operators will be
+	appended.
+
+	ahxencode: If 0, then image streams will be send as binary,
+	otherwise they will be asciihexencoded.
+*/
 pdf_processor *pdf_new_buffer_processor(fz_context *ctx, fz_buffer *buffer, int ahxencode);
 
+/*
+	Create an output processor. This
+	sends the incoming PDF operator stream to an fz_output stream.
+
+	out: The output stream to which operators will be sent.
+
+	ahxencode: If 0, then image streams will be send as binary,
+	otherwise they will be asciihexencoded.
+*/
 pdf_processor *pdf_new_output_processor(fz_context *ctx, fz_output *out, int ahxencode);
 
-pdf_processor *pdf_new_filter_processor(fz_context *ctx, pdf_document *doc, pdf_processor *chain, pdf_obj *old_res, pdf_obj *new_res);
+/*
+	opaque: Opaque value that is passed to all the filter functions.
 
-typedef int (pdf_text_filter_fn)(fz_context *ctx, void *opaque, int *ucsbuf, int ucslen, fz_matrix trm, fz_matrix ctm, fz_rect bbox);
+	image_filter: A function called to assess whether a given
+	image should be removed or not.
 
-typedef void (pdf_after_text_object_fn)(fz_context *ctx, void *opaque, pdf_document *doc, pdf_processor *chain, fz_matrix ctm);
+	text_filter: A function called to assess whether a given
+	character should be removed or not.
 
-pdf_processor *
-pdf_new_filter_processor_with_text_filter(fz_context *ctx, pdf_document *doc, int structparents, pdf_processor *chain, pdf_obj *old_rdb, pdf_obj *new_rdb, pdf_text_filter_fn *text_filter, pdf_after_text_object_fn *after, void *text_filter_opaque);
+	after_text_object: A function called after each text object.
+	This allows the caller to insert some extra content if
+	desired.
 
+	end_page: A function called at the end of a page.
+	This allows the caller to insert some extra content after
+	all other content.
+
+	sanitize: If false, will only clean the syntax. This disables all filtering!
+
+	recurse: Clean/sanitize/filter resources recursively.
+
+	instance_forms: Always recurse on XObject Form resources, but will
+	create a new instance of each XObject Form that is used, filtered
+	individually.
+
+	ascii: If true, escape all binary data in the output.
+*/
+typedef struct
+{
+	void *opaque;
+	fz_image *(*image_filter)(fz_context *ctx, void *opaque, fz_matrix ctm, const char *name, fz_image *image);
+	int (*text_filter)(fz_context *ctx, void *opaque, int *ucsbuf, int ucslen, fz_matrix trm, fz_matrix ctm, fz_rect bbox);
+	void (*after_text_object)(fz_context *ctx, void *opaque, pdf_document *doc, pdf_processor *chain, fz_matrix ctm);
+	void (*end_page)(fz_context *ctx, fz_buffer *buffer, void *arg);
+
+	int recurse;
+	int instance_forms;
+	int sanitize;
+	int ascii;
+} pdf_filter_options;
+
+/*
+	Create a filter processor. This filters the PDF operators
+	it is fed, and passes them down (with some changes) to the
+	child filter.
+
+	The changes made by the filter are:
+
+	* No operations are allowed to change the top level gstate.
+	Additional q/Q operators are inserted to prevent this.
+
+	* Repeated/unnecessary colour operators are removed (so,
+	for example, "0 0 0 rg 0 1 rg 0.5 g" would be sanitised to
+	"0.5 g")
+
+	The intention of these changes is to provide a simpler,
+	but equivalent stream, repairing problems with mismatched
+	operators, maintaining structure (such as BMC, EMC calls)
+	and leaving the graphics state in an known (default) state
+	so that subsequent operations (such as synthesising new
+	operators to be appended to the stream) are easier.
+
+	The net graphical effect of the filtered operator stream
+	should be identical to the incoming operator stream.
+
+	chain: The child processor to which the filtered operators
+	will be fed.
+
+	old_res: The incoming resource dictionary.
+
+	new_res: An (initially empty) resource dictionary that will
+	be populated by copying entries from the old dictionary to
+	the new one as they are used. At the end therefore, this
+	contains exactly those resource objects actually required.
+
+	The filter options struct allows you to filter objects using callbacks.
+*/
+pdf_processor *pdf_new_filter_processor(fz_context *ctx, pdf_document *doc, pdf_processor *chain, pdf_obj *old_res, pdf_obj *new_res, int struct_parents, fz_matrix transform, pdf_filter_options *filter);
+pdf_obj *pdf_filter_xobject_instance(fz_context *ctx, pdf_obj *old_xobj, pdf_obj *page_res, fz_matrix ctm, pdf_filter_options *filter, pdf_cycle_list *cycle_up);
+
+/*
+	Functions to actually process annotations, glyphs and general stream objects.
+*/
 void pdf_process_contents(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *obj, pdf_obj *res, fz_cookie *cookie);
-void pdf_process_annot(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_page *page, pdf_annot *annot, fz_cookie *cookie);
+void pdf_process_annot(fz_context *ctx, pdf_processor *proc, pdf_annot *annot, fz_cookie *cookie);
 void pdf_process_glyph(fz_context *ctx, pdf_processor *proc, pdf_document *doc, pdf_obj *resources, fz_buffer *contents);
 
 /* Text handling helper functions */
-typedef struct pdf_text_state_s
+typedef struct
 {
 	float char_space;
 	float word_space;
@@ -197,7 +315,7 @@ typedef struct pdf_text_state_s
 	float rise;
 } pdf_text_state;
 
-typedef struct pdf_text_object_state_s
+typedef struct
 {
 	fz_text *text;
 	fz_rect text_bbox;
